@@ -6,16 +6,26 @@ import datetime
 import json
 import os
 import sys
+import textwrap
 from collections import defaultdict
 from neuro_sdk import (
     Action,
     Client,
+    DiskVolume,
+    EnvParseResult,
     HTTPPort,
     IllegalArgumentError,
+    JobDescription,
+    JobRestartPolicy,
     JobStatus,
     Permission,
+    Preset,
     RemoteImage,
     ResourceNotFound,
+    SecretFile,
+    Tag,
+    Volume,
+    VolumeParseResult,
 )
 from rich.console import Console
 from rich.panel import Panel
@@ -28,8 +38,10 @@ from typing import (
     Dict,
     Generic,
     Iterable,
+    List,
     Mapping,
     Optional,
+    Sequence,
     Set,
     Tuple,
     TypeVar,
@@ -61,6 +73,7 @@ from .storage import (
     Bake,
     BakeImage,
     FinishedTask,
+    RetryReadStorage,
     StartedTask,
     Storage,
     _dt2str,
@@ -69,9 +82,11 @@ from .types import AlwaysT, FullID, ImageStatus, RemotePath, TaskStatus
 from .utils import (
     TERMINATED_JOB_STATUSES,
     TERMINATED_TASK_STATUSES,
+    RetryConfig,
     fmt_id,
     fmt_raw_id,
     fmt_status,
+    retry,
 )
 
 
@@ -124,29 +139,31 @@ async def get_running_flow(
         early_images={},
     )
     async for image in storage.list_bake_images(bake):
-        sub = local_info
-        for part in image.prefix:
-            if part not in sub.children_info:
-                new_sub = LocallyPreparedInfo(
-                    children_info={},
-                    early_images={},
-                )
-                assert isinstance(sub.children_info, dict)
-                sub.children_info[part] = new_sub
-            sub = sub.children_info[part]
-        assert isinstance(sub.early_images, dict)
-        dockerfile = None
-        if image.context_on_storage and image.dockerfile_rel:
-            dockerfile = image.context_on_storage / str(image.dockerfile_rel)
-        sub.early_images[image.yaml_id] = EarlyImageCtx(
-            id=image.yaml_id,
-            ref=image.ref,
-            context=image.context_on_storage,
-            dockerfile=dockerfile,
-            dockerfile_rel=RemotePath(image.dockerfile_rel)
-            if image.dockerfile_rel
-            else None,
-        )
+        for yaml_def in image.yaml_defs:
+            *prefix, image_id = yaml_def
+            sub = local_info
+            for part in prefix:
+                if part not in sub.children_info:
+                    new_sub = LocallyPreparedInfo(
+                        children_info={},
+                        early_images={},
+                    )
+                    assert isinstance(sub.children_info, dict)
+                    sub.children_info[part] = new_sub
+                sub = sub.children_info[part]
+            assert isinstance(sub.early_images, dict)
+            dockerfile = None
+            if image.context_on_storage and image.dockerfile_rel:
+                dockerfile = image.context_on_storage / str(image.dockerfile_rel)
+            sub.early_images[image_id] = EarlyImageCtx(
+                id=image_id,
+                ref=image.ref,
+                context=image.context_on_storage,
+                dockerfile=dockerfile,
+                dockerfile_rel=RemotePath(image.dockerfile_rel)
+                if image.dockerfile_rel
+                else None,
+            )
     return await RunningBatchFlow.create(
         config_loader=config_loader,
         batch=bake.batch,
@@ -199,7 +216,6 @@ class Graph(Generic[_T]):
             yield ready, self._metas[ready[:-1]]
 
     def get_graph_data(self, node: FullID) -> _T:
-        assert node in self._topos, f"Graph {node} not found"
         return self._metas[node]
 
     def get_meta(self, node: FullID) -> _T:
@@ -331,20 +347,146 @@ class BakeTasksManager:
         return dep.state
 
 
+class ImageBuildCommandError(Exception):
+    def __init__(self, exit_code: int, stdout: str, stderr: str) -> None:
+        self.exit_code = exit_code
+        self.stdout = stdout
+        self.stderr = stderr
+
+    def __str__(self) -> str:
+        return (
+            f"Image builder command failed:\n"
+            f"exit code: {self.exit_code}\n"
+            f"stdout: \n{textwrap.indent(self.stdout, '  ')}\n"
+            f"stderr: \n{textwrap.indent(self.stderr, '  ')}"
+        )
+
+
 async def start_image_build(*cmd: str) -> str:
     subprocess = await asyncio.create_subprocess_exec(
         *cmd,
         stdout=asyncio.subprocess.PIPE,
         stderr=asyncio.subprocess.PIPE,
     )
-    while True:
-        assert subprocess.stdout
+    assert subprocess.stdout
+    assert subprocess.stderr
+    stdout = ""
+    while subprocess.returncode is None:
         line = (await subprocess.stdout.readline()).decode()
+        stdout += line
         if line.startswith("Job ID: "):
             # Job is started, we can freely kill neuro-cli
             subprocess.terminate()
             return line[len("Job ID: ") :].strip()
 
+    stdout += (await subprocess.stdout.read()).decode()
+    stderr = (await subprocess.stderr.read()).decode()
+    raise ImageBuildCommandError(
+        exit_code=subprocess.returncode,
+        stdout=stdout,
+        stderr=stderr,
+    )
+
+
+class RetryReadNeuroClient(RetryConfig):
+    def __init__(self, client: Client) -> None:
+        super().__init__()
+        self._client = client
+
+    async def job_start(
+        self,
+        *,
+        image: RemoteImage,
+        preset_name: str,
+        entrypoint: Optional[str] = None,
+        command: Optional[str] = None,
+        working_dir: Optional[str] = None,
+        http: Optional[HTTPPort] = None,
+        env: Optional[Mapping[str, str]] = None,
+        volumes: Sequence[Volume] = (),
+        secret_env: Optional[Mapping[str, URL]] = None,
+        secret_files: Sequence[SecretFile] = (),
+        disk_volumes: Sequence[DiskVolume] = (),
+        tty: bool = False,
+        shm: bool = False,
+        name: Optional[str] = None,
+        tags: Sequence[str] = (),
+        description: Optional[str] = None,
+        pass_config: bool = False,
+        wait_for_jobs_quota: bool = False,
+        schedule_timeout: Optional[float] = None,
+        restart_policy: JobRestartPolicy = JobRestartPolicy.NEVER,
+        life_span: Optional[float] = None,
+        privileged: bool = False,
+    ) -> JobDescription:
+        return await self._client.jobs.start(
+            image=image,
+            preset_name=preset_name,
+            entrypoint=entrypoint,
+            command=command,
+            working_dir=working_dir,
+            http=http,
+            env=env,
+            volumes=volumes,
+            secret_env=secret_env,
+            secret_files=secret_files,
+            disk_volumes=disk_volumes,
+            tty=tty,
+            shm=shm,
+            name=name,
+            tags=tags,
+            description=description,
+            pass_config=pass_config,
+            wait_for_jobs_quota=wait_for_jobs_quota,
+            schedule_timeout=schedule_timeout,
+            restart_policy=restart_policy,
+            life_span=life_span,
+            privileged=privileged,
+        )
+
+    @retry
+    async def job_status(self, raw_id: str) -> JobDescription:
+        return await self._client.jobs.status(raw_id)
+
+    @retry
+    async def _job_logs(self, raw_id: str) -> List[bytes]:
+        chunks = []
+        async for chunk in self._client.jobs.monitor(raw_id):
+            chunks.append(chunk)
+        return chunks
+
+    async def job_logs(self, raw_id: str) -> AsyncIterator[bytes]:
+        for chunk in await self._job_logs(raw_id):
+            yield chunk
+
+    async def job_kill(self, raw_id: str) -> None:
+        await self._client.jobs.kill(raw_id)
+
+    @retry
+    async def image_tag_info(self, remote_image: RemoteImage) -> Tag:
+        return await self._client.images.tag_info(remote_image)
+
+    def parse_remote_image(self, image: str) -> RemoteImage:
+        return self._client.parse.remote_image(image)
+
+    @property
+    def config_presets(self) -> Mapping[str, Preset]:
+        return self._client.config.presets
+
+    def parse_envs(
+        self, env: Sequence[str], env_file: Sequence[str] = ()
+    ) -> EnvParseResult:
+        return self._client.parse.envs(env, env_file)
+
+    def parse_volumes(self, volume: Sequence[str]) -> VolumeParseResult:
+        return self._client.parse.volumes(volume)
+
+    async def user_add(self, role_name: str) -> None:
+        await self._client.users.add(role_name)
+
+    async def user_share(self, user: str, permission: Permission) -> Permission:
+        return await self._client.users.share(user, permission)
+
 
 class BatchExecutor:
     def __init__(
@@ -352,7 +494,7 @@ class BatchExecutor:
         console: Console,
         flow: RunningBatchFlow,
         attempt: Attempt,
-        client: Client,
+        client: RetryReadNeuroClient,
         storage: Storage,
         *,
         polling_timeout: float = 1,
@@ -405,6 +547,8 @@ class BatchExecutor:
         project_role: Optional[str] = None,
         run_builder_job: Callable[..., Awaitable[str]] = start_image_build,
     ) -> AsyncIterator["BatchExecutor"]:
+        storage = RetryReadStorage(storage)
+
         console.log("Fetch bake data")
         bake = await storage.fetch_bake(
             executor_data.project,
@@ -424,7 +568,7 @@ class BatchExecutor:
             console,
             flow,
             attempt,
-            client,
+            RetryReadNeuroClient(client),
             storage,
             polling_timeout=polling_timeout,
             transient_progress=transient_progress,
@@ -444,8 +588,11 @@ class BatchExecutor:
         pass
 
     async def _refresh_attempt(self) -> None:
+        # TODO: Drop force_no_cache when storage is refactored
         self._attempt = await self._storage.find_attempt(
-            self._attempt.bake, self._attempt.number
+            self._attempt.bake,
+            self._attempt.number,
+            force_no_cache=True,
         )
 
     def _only_completed_needs(
@@ -492,9 +639,45 @@ class BatchExecutor:
         )
         return await flow.get_action(tid, needs=needs)
 
-    async def _get_image(self, prefix: FullID, image_id: str) -> ImageCtx:
-        flow = self._graphs.get_graph_data(prefix)
-        return flow.images[image_id]
+    async def _get_image(self, bake_image: BakeImage) -> Optional[ImageCtx]:
+        actions = []
+        full_id_to_image: Dict[FullID, ImageCtx] = {}
+        for yaml_def in bake_image.yaml_defs:
+            *prefix, image_id = yaml_def
+            actions.append(".".join(prefix))
+            try:
+                flow = self._graphs.get_graph_data(tuple(prefix))
+            except KeyError:
+                pass
+            else:
+                full_id_to_image[yaml_def] = flow.images[image_id]
+        image_ctx = None
+        for _it in full_id_to_image.values():
+            image_ctx = _it  # Return any context
+            break
+        if not all(image_ctx == it for it in full_id_to_image.values()):
+            locations_str = "\n".join(
+                f" - {'.'.join(full_id)}" for full_id in full_id_to_image.keys()
+            )
+            self._progress.log(
+                f"[red]Warning:[/red] definitions for image with"
+                f" ref [b]{bake_image.ref}[/b] differ:\n"
+                f"{locations_str}"
+            )
+        if len(full_id_to_image.values()) == 0:
+            actions_str = ", ".join(f"'{it}'" for it in actions)
+            self._progress.log(
+                f"[red]Warning:[/red] image with ref "
+                f"[b]{bake_image.ref}[/b] is referred "
+                + (
+                    f"before action {actions_str} "
+                    "that holds its definition was able to run"
+                    if len(actions) == 1
+                    else f"before actions {actions_str} "
+                    "that hold its definition were able to run"
+                )
+            )
+        return image_ctx
 
     # Graph helpers
 
@@ -604,7 +787,7 @@ class BatchExecutor:
             if not st.raw_id:
                 continue
             try:
-                job_descr = await self._client.jobs.status(st.raw_id)
+                job_descr = await self._client.job_status(st.raw_id)
             except ResourceNotFound:
                 pass
             else:
@@ -717,6 +900,13 @@ class BatchExecutor:
             ),
             justify="center",
         )
+        if attempt_status != TaskStatus.SUCCEEDED:
+            self._progress.log(
+                "[blue b]Hint:[/blue b] you can restart bake starting "
+                "from first failed task "
+                "by the following command:\n"
+                f"[b]neuro-flow restart {self._attempt.bake.bake_id}[/b]"
+            )
 
     async def _cancel_unfinished(self) -> None:
         self._is_cancelling = True
@@ -724,11 +914,11 @@ class BatchExecutor:
             task = await self._get_task(full_id)
             if task.enable is not AlwaysT():
                 self._progress.log(f"Task {fmt_id(st.id)} is being killed")
-                await self._client.jobs.kill(st.raw_id)
+                await self._client.job_kill(st.raw_id)
         async for image in self._storage.list_bake_images(self._attempt.bake):
             if image.status == ImageStatus.BUILDING:
                 assert image.builder_job_id
-                await self._client.jobs.kill(image.builder_job_id)
+                await self._client.job_kill(image.builder_job_id)
 
     async def _store_to_cache(self, ft: FinishedTask) -> None:
         task = await self._get_task(ft.id)
@@ -743,7 +933,7 @@ class BatchExecutor:
     async def _process_started(self) -> bool:
         # Process tasks
         for full_id, st in self._tasks_mgr.unfinished_tasks.items():
-            job_descr = await self._client.jobs.status(st.raw_id)
+            job_descr = await self._client.job_status(st.raw_id)
             if (
                 job_descr.status in {JobStatus.RUNNING, JobStatus.SUCCEEDED}
                 and full_id not in self._tasks_mgr.running
@@ -758,7 +948,7 @@ class BatchExecutor:
                 )
             if job_descr.status in TERMINATED_JOB_STATUSES:
                 async with CmdProcessor() as proc:
-                    async for chunk in self._client.jobs.monitor(st.raw_id):
+                    async for chunk in self._client.job_logs(st.raw_id):
                         async for line in proc.feed_chunk(chunk):
                             pass
                     async for line in proc.feed_eof():
@@ -837,13 +1027,13 @@ class BatchExecutor:
 
     async def _is_image_in_registry(self, remote_image: RemoteImage) -> bool:
         try:
-            await self._client.images.tag_info(remote_image)
+            await self._client.image_tag_info(remote_image)
         except ResourceNotFound:
             return False
         return True
 
     async def _start_task(self, full_id: FullID, task: Task) -> Optional[StartedTask]:
-        remote_image = self._client.parse.remote_image(task.image)
+        remote_image = self._client.parse_remote_image(task.image)
         if remote_image.cluster_name is None:  # Not a neuro registry image
             return await self._run_task(full_id, task)
         try:
@@ -854,7 +1044,10 @@ class BatchExecutor:
             # Not defined in the bake
             return await self._run_task(full_id, task)
 
-        image_ctx = await self._get_image(bake_image.prefix, bake_image.yaml_id)
+        image_ctx = await self._get_image(bake_image)
+        if image_ctx is None:
+            # The image referred before definition
+            return await self._run_task(full_id, task)
 
         if not image_ctx.force_rebuild and await self._is_image_in_registry(
             remote_image
@@ -862,7 +1055,7 @@ class BatchExecutor:
             return await self._run_task(full_id, task)
 
         if bake_image.status == ImageStatus.PENDING:
-            await self._start_image_build(bake_image)
+            await self._start_image_build(bake_image, image_ctx)
             return None  # wait for next pulling interval
         elif bake_image.status == ImageStatus.BUILDING:
             return None  # wait for next pulling interval
@@ -873,29 +1066,27 @@ class BatchExecutor:
     async def _run_task(self, full_id: FullID, task: Task) -> StartedTask:
         preset_name = task.preset
         if preset_name is None:
-            preset_name = next(iter(self._client.config.presets))
+            preset_name = next(iter(self._client.config_presets))
 
-        env_dict, secret_env_dict = self._client.parse.env(
-            [f"{k}={v}" for k, v in task.env.items()]
-        )
+        envs = self._client.parse_envs([f"{k}={v}" for k, v in task.env.items()])
 
-        volumes_parsed = self._client.parse.volumes(task.volumes)
+        volumes_parsed = self._client.parse_volumes(task.volumes)
         volumes = list(volumes_parsed.volumes)
 
         http_auth = task.http_auth
         if http_auth is None:
             http_auth = HTTPPort.requires_auth
 
-        job = await self._client.jobs.start(
+        job = await self._client.job_start(
             shm=True,
             tty=False,
-            image=self._client.parse.remote_image(task.image),
+            image=self._client.parse_remote_image(task.image),
             preset_name=preset_name,
             entrypoint=task.entrypoint,
             command=task.cmd,
             http=HTTPPort(task.http_port, http_auth) if task.http_port else None,
-            env=env_dict,
-            secret_env=secret_env_dict,
+            env=envs.env,
+            secret_env=envs.secret_env,
             volumes=volumes,
             working_dir=str(task.workdir) if task.workdir else None,
             secret_files=list(volumes_parsed.secret_files),
@@ -910,8 +1101,9 @@ class BatchExecutor:
         await self._add_resource(job.uri)
         return await self._storage.start_task(self._attempt, full_id, job)
 
-    async def _start_image_build(self, bake_image: BakeImage) -> None:
-        image_ctx = await self._get_image(bake_image.prefix, bake_image.yaml_id)
+    async def _start_image_build(
+        self, bake_image: BakeImage, image_ctx: ImageCtx
+    ) -> None:
         context = bake_image.context_on_storage or image_ctx.context
         dockerfile_rel = bake_image.dockerfile_rel or image_ctx.dockerfile_rel
 
@@ -938,7 +1130,6 @@ class BatchExecutor:
             cmd.append(f"--preset={image_ctx.build_preset}")
         cmd.append(str(context))
         cmd.append(str(bake_image.ref))
-
         builder_job_id = await self._run_builder_job(*cmd)
         await self._storage.update_bake_image(
             self._attempt.bake,
@@ -953,7 +1144,7 @@ class BatchExecutor:
         async for image in self._storage.list_bake_images(self._attempt.bake):
             if image.status == ImageStatus.BUILDING:
                 assert image.builder_job_id
-                descr = await self._client.jobs.status(image.builder_job_id)
+                descr = await self._client.job_status(image.builder_job_id)
                 if descr.status == JobStatus.SUCCEEDED:
                     await self._storage.update_bake_image(
                         self._attempt.bake,
@@ -977,7 +1168,7 @@ class BatchExecutor:
         if self._is_projet_role_created:
             return
         try:
-            await self._client.users.add(project_role)
+            await self._client.user_add(project_role)
         except IllegalArgumentError as e:
             if "already exists" not in str(e):
                 raise
@@ -990,7 +1181,7 @@ class BatchExecutor:
         await self._create_project_role(project_role)
         permission = Permission(uri, Action.WRITE)
         try:
-            await self._client.users.share(project_role, permission)
+            await self._client.user_share(project_role, permission)
         except ValueError:
             self._progress.log(f"[red]Cannot share [b]{uri!s}[/b]")
 
