@@ -9,7 +9,7 @@ from typing import Callable, Generic, List, Optional, Sequence, Tuple, TypeVar,
 from neuro_flow.batch_runner import BatchRunner
 from neuro_flow.cli.root import Root
 from neuro_flow.live_runner import LiveRunner
-from neuro_flow.storage import FSStorage, NeuroStorageFS, Storage
+from neuro_flow.storage import APIStorage, NeuroStorageFS, Storage
 
 
 if sys.version_info >= (3, 7):
@@ -22,7 +22,10 @@ _T = TypeVar("_T")
 
 class AsyncType(click.ParamType, Generic[_T], abc.ABC):
     def convert(
-        self, value: str, param: Optional[click.Parameter], ctx: Optional[click.Context]
+        self,
+        value: str,
+        param: Optional[click.Parameter],
+        ctx: Optional[click.Context],
     ) -> _T:
         assert ctx is not None
         root = cast(Root, ctx.obj)
@@ -48,7 +51,11 @@ class AsyncType(click.ParamType, Generic[_T], abc.ABC):
 
     @abc.abstractmethod
     async def async_complete(
-        self, root: Root, ctx: click.Context, args: Sequence[str], incomplete: str
+        self,
+        root: Root,
+        ctx: click.Context,
+        args: Sequence[str],
+        incomplete: str,
     ) -> List[Tuple[str, Optional[str]]]:
         pass
 
@@ -78,10 +85,10 @@ class LiveJobType(AsyncType[str]):
         async with AsyncExitStack() as stack:
             client = await stack.enter_async_context(neuro_sdk.get())
             storage: Storage = await stack.enter_async_context(
-                FSStorage(NeuroStorageFS(client))
+                APIStorage(client, NeuroStorageFS(client))
             )
             runner = await stack.enter_async_context(
-                LiveRunner(root.config_dir, root.console, client, storage)
+                LiveRunner(root.config_dir, root.console, client, storage, root)
             )
             variants = list(runner.flow.job_ids)
             if self._allow_all:
@@ -121,10 +128,10 @@ class LiveJobSuffixType(AsyncType[str]):
         async with AsyncExitStack() as stack:
             client = await stack.enter_async_context(neuro_sdk.get())
             storage: Storage = await stack.enter_async_context(
-                FSStorage(NeuroStorageFS(client))
+                APIStorage(client, NeuroStorageFS(client))
             )
             runner = await stack.enter_async_context(
-                LiveRunner(root.config_dir, root.console, client, storage)
+                LiveRunner(root.config_dir, root.console, client, storage, root)
             )
             return [
                 (suffix, None)
@@ -161,10 +168,10 @@ class LiveImageType(AsyncType[str]):
         async with AsyncExitStack() as stack:
             client = await stack.enter_async_context(neuro_sdk.get())
             storage: Storage = await stack.enter_async_context(
-                FSStorage(NeuroStorageFS(client))
+                APIStorage(client, NeuroStorageFS(client))
             )
             runner = await stack.enter_async_context(
-                LiveRunner(root.config_dir, root.console, client, storage)
+                LiveRunner(root.config_dir, root.console, client, storage, root)
             )
             variants = [
                 image
@@ -204,10 +211,10 @@ class LiveVolumeType(AsyncType[str]):
         async with AsyncExitStack() as stack:
             client = await stack.enter_async_context(neuro_sdk.get())
             storage: Storage = await stack.enter_async_context(
-                FSStorage(NeuroStorageFS(client))
+                APIStorage(client, NeuroStorageFS(client))
             )
             runner = await stack.enter_async_context(
-                LiveRunner(root.config_dir, root.console, client, storage)
+                LiveRunner(root.config_dir, root.console, client, storage, root)
             )
             variants = [
                 volume.id
@@ -282,14 +289,16 @@ class BakeType(AsyncType[str]):
         async with AsyncExitStack() as stack:
             client = await stack.enter_async_context(neuro_sdk.get())
             storage: Storage = await stack.enter_async_context(
-                FSStorage(NeuroStorageFS(client))
+                APIStorage(client, NeuroStorageFS(client))
             )
             runner: BatchRunner = await stack.enter_async_context(
-                BatchRunner(root.config_dir, root.console, client, storage)
+                BatchRunner(root.config_dir, root.console, client, storage, root)
             )
             try:
                 async for bake in runner.get_bakes():
                     variants.append(bake.bake_id)
+                    if bake.name is not None:
+                        variants.append(bake.name)
             except ValueError:
                 pass
         return [(bake, None) for bake in variants if bake.startswith(incomplete)]
@@ -336,10 +345,10 @@ class BakeTaskType(AsyncType[str]):
         async with AsyncExitStack() as stack:
             client = await stack.enter_async_context(neuro_sdk.get())
             storage: Storage = await stack.enter_async_context(
-                FSStorage(NeuroStorageFS(client))
+                APIStorage(client, NeuroStorageFS(client))
             )
             runner: BatchRunner = await stack.enter_async_context(
-                BatchRunner(root.config_dir, root.console, client, storage)
+                BatchRunner(root.config_dir, root.console, client, storage, root)
             )
             attempt = await runner.get_bake_attempt(bake_id, attempt_no=attempt_no)
             started, finished = await storage.fetch_attempt(attempt)
