@@ -6,7 +6,7 @@ from neuro_flow.cli.click_types import LIVE_VOLUME_OR_ALL
 from neuro_flow.cli.root import Root
 from neuro_flow.cli.utils import argument, wrap_async
 from neuro_flow.live_runner import LiveRunner
-from neuro_flow.storage import FSStorage, NeuroStorageFS, Storage
+from neuro_flow.storage import APIStorage, NeuroStorageFS, Storage
 
 
 if sys.version_info >= (3, 7):
@@ -29,10 +29,10 @@ async def upload(
     async with AsyncExitStack() as stack:
         client = await stack.enter_async_context(neuro_sdk.get())
         storage: Storage = await stack.enter_async_context(
-            FSStorage(NeuroStorageFS(client))
+            APIStorage(client, NeuroStorageFS(client))
         )
         runner = await stack.enter_async_context(
-            LiveRunner(root.config_dir, root.console, client, storage)
+            LiveRunner(root.config_dir, root.console, client, storage, root)
         )
         if volume != "ALL":
             await runner.upload(volume)
@@ -54,10 +54,10 @@ async def download(
     async with AsyncExitStack() as stack:
         client = await stack.enter_async_context(neuro_sdk.get())
         storage: Storage = await stack.enter_async_context(
-            FSStorage(NeuroStorageFS(client))
+            APIStorage(client, NeuroStorageFS(client))
         )
         runner = await stack.enter_async_context(
-            LiveRunner(root.config_dir, root.console, client, storage)
+            LiveRunner(root.config_dir, root.console, client, storage, root)
         )
         if volume != "ALL":
             await runner.download(volume)
@@ -79,10 +79,10 @@ async def clean(
     async with AsyncExitStack() as stack:
         client = await stack.enter_async_context(neuro_sdk.get())
         storage: Storage = await stack.enter_async_context(
-            FSStorage(NeuroStorageFS(client))
+            APIStorage(client, NeuroStorageFS(client))
         )
         runner = await stack.enter_async_context(
-            LiveRunner(root.config_dir, root.console, client, storage)
+            LiveRunner(root.config_dir, root.console, client, storage, root)
         )
         if volume != "ALL":
             await runner.clean(volume)
@@ -99,9 +99,9 @@ async def mkvolumes(
     async with AsyncExitStack() as stack:
         client = await stack.enter_async_context(neuro_sdk.get())
         storage: Storage = await stack.enter_async_context(
-            FSStorage(NeuroStorageFS(client))
+            APIStorage(client, NeuroStorageFS(client))
         )
         runner = await stack.enter_async_context(
-            LiveRunner(root.config_dir, root.console, client, storage)
+            LiveRunner(root.config_dir, root.console, client, storage, root)
         )
         await runner.mkvolumes()
