@@ -345,7 +345,9 @@ def parse_dict(
     field_names = set()
     for f in dataclasses.fields(res_type):
         field_names.add(f.name)
-        if f.name not in found_fields:
+        if f.name == "_specified_fields":
+            optional_fields[f.name] = set(data.keys())
+        elif f.name not in found_fields:
             key = f.name
             item_ctor = keys[key]
             if (
@@ -425,79 +427,6 @@ def parse_cache(ctor: BaseConstructor, node: yaml.MappingNode) -> ast.Cache:
     )
 
 
-# #### Project parser ####
-
-
-class ProjectLoader(Reader, Scanner, Parser, Composer, BaseConstructor, BaseResolver):
-    def __init__(self, stream: TextIO) -> None:
-        Reader.__init__(self, stream)
-        Scanner.__init__(self)
-        Parser.__init__(self)
-        Composer.__init__(self)
-        BaseConstructor.__init__(self)
-        BaseResolver.__init__(self)
-
-
-PROJECT = {"id": SimpleIdExpr, "owner": SimpleOptStrExpr, "role": SimpleOptStrExpr}
-
-
-def parse_project_main(ctor: BaseConstructor, node: yaml.MappingNode) -> ast.Project:
-    ret = parse_dict(
-        ctor,
-        node,
-        PROJECT,
-        ast.Project,
-    )
-    return ret
-
-
-ProjectLoader.add_path_resolver("project:main", [])  # type: ignore
-ProjectLoader.add_constructor("project:main", parse_project_main)  # type: ignore
-
-
-def parse_project_stream(stream: TextIO) -> ast.Project:
-    ret: ast.Project
-    loader = ProjectLoader(stream)
-    try:
-        ret = loader.get_single_data()
-        assert isinstance(ret, ast.Project)
-        return ret
-    finally:
-        loader.dispose()  # type: ignore[no-untyped-call]
-
-
-def make_default_project(workspace_stem: str) -> ast.Project:
-    project_id = workspace_stem.replace("-", "_")
-    if not project_id.isidentifier():
-        raise ValueError(
-            f'Workspace directory name "{workspace_stem}" is invalid identifier'
-        )
-    if project_id == project_id.upper():
-        raise ValueError(
-            f'Workspace directory name "{workspace_stem}" is invalid '
-            "identifier, uppercase names are reserved for internal usage"
-        )
-    return ast.Project(
-        _start=Pos(0, 0, LocalPath("<default>")),
-        _end=Pos(0, 0, LocalPath("<default>")),
-        id=SimpleIdExpr(
-            Pos(0, 0, LocalPath("<default>")),
-            Pos(0, 0, LocalPath("<default>")),
-            project_id,
-        ),
-        owner=SimpleOptStrExpr(
-            Pos(0, 0, LocalPath("<default>")),
-            Pos(0, 0, LocalPath("<default>")),
-            None,
-        ),
-        role=SimpleOptStrExpr(
-            Pos(0, 0, LocalPath("<default>")),
-            Pos(0, 0, LocalPath("<default>")),
-            None,
-        ),
-    )
-
-
 # #### Flow parser ####
 
 
@@ -689,7 +618,7 @@ FlowLoader.add_constructor("flow:cache", parse_cache)  # type: ignore
 EXEC_UNIT = {
     "title": OptStrExpr,
     "name": OptStrExpr,
-    "image": StrExpr,
+    "image": OptStrExpr,
     "preset": OptStrExpr,
     "schedule_timeout": OptTimeDeltaExpr,
     "entrypoint": OptStrExpr,
@@ -706,14 +635,24 @@ EXEC_UNIT = {
     "pass_config": OptBoolExpr,
 }
 
-
-JOB = {
+JOB_MIXIN = {
+    **EXEC_UNIT,
     "detach": OptBoolExpr,
     "browse": OptBoolExpr,
     "port_forward": ExprOrSeq(PortPairExpr, port_pair_item),
     "multi": SimpleOptBoolExpr,
     "params": None,
+    "inherits": SimpleSeq(StrExpr),
+}
+
+JOB: Dict[str, Any] = {
     **EXEC_UNIT,
+    "detach": OptBoolExpr,
+    "browse": OptBoolExpr,
+    "port_forward": ExprOrSeq(PortPairExpr, port_pair_item),
+    "multi": SimpleOptBoolExpr,
+    "params": None,
+    "inherits": SimpleSeq(StrExpr),
 }
 
 JOB_ACTION_CALL = {
@@ -818,9 +757,44 @@ def parse_jobs(
     return ret
 
 
+def parse_mixin(
+    ctor: FlowLoader, node: yaml.MappingNode
+) -> Union[ast.JobMixin, ast.TaskMixin]:
+    if ctor._kind == ast.FlowKind.LIVE:
+        return parse_dict(
+            ctor,
+            node,
+            JOB_MIXIN,
+            ast.JobMixin,
+        )
+    elif ctor._kind == ast.FlowKind.BATCH:
+        return parse_dict(
+            ctor,
+            node,
+            TASK_MIXIN,
+            ast.TaskMixin,
+        )
+    else:
+        raise ValueError(f"Unknown kind {ctor._kind}")
+
+
+def parse_mixins(
+    ctor: FlowLoader, node: yaml.MappingNode
+) -> Dict[str, Union[ast.JobMixin, ast.TaskMixin]]:
+    ret = {}
+    for k, v in node.value:
+        key = ctor.construct_id(k)
+        value = parse_mixin(ctor, v)
+        ret[key] = value
+    return ret
+
+
 FlowLoader.add_path_resolver("flow:jobs", [(dict, "jobs")])  # type: ignore
 FlowLoader.add_constructor("flow:jobs", parse_jobs)  # type: ignore
 
+FlowLoader.add_path_resolver("flow:mixins", [(dict, "mixins")])  # type: ignore
+FlowLoader.add_constructor("flow:mixins", parse_mixins)  # type: ignore
+
 TASK_BASE = {
     "id": OptIdExpr,
     "needs": None,
@@ -832,6 +806,16 @@ TASK_BASE = {
 TASK: Mapping[str, Any] = {
     **TASK_BASE,
     **EXEC_UNIT,
+    "inherits": SimpleSeq(StrExpr),
+}
+
+TASK_MIXIN: Mapping[str, Any] = {
+    **EXEC_UNIT,
+    "needs": None,
+    "strategy": None,
+    "enable": EnableExpr,
+    "cache": None,
+    "inherits": SimpleSeq(StrExpr),
 }
 
 
@@ -888,6 +872,7 @@ def parse_task(ctor: BaseConstructor, node: yaml.MappingNode) -> ast.Base:
         ast.Base,
         preprocess=select_task_or_action_or_module,
         find_res_type=find_task_type,
+        ret_name="Task",
     )
 
 
@@ -951,6 +936,7 @@ FLOW = {
     "id": SimpleOptIdExpr,
     "title": SimpleOptStrExpr,
     "life_span": OptTimeDeltaExpr,
+    "mixins": None,
     "images": None,
     "volumes": None,
     "defaults": None,
@@ -1467,3 +1453,126 @@ def parse_bake_meta(meta_file: LocalPath) -> Mapping[str, str]:
         result = BakeMetaLoader(f).get_single_data()
         assert isinstance(result, dict)
         return result
+
+
+# #### Project parser ####
+
+
+class ProjectLoader(Reader, Scanner, Parser, Composer, BaseConstructor, BaseResolver):
+    def __init__(self, stream: TextIO) -> None:
+        Reader.__init__(self, stream)
+        Scanner.__init__(self)
+        Parser.__init__(self)
+        Composer.__init__(self)
+        BaseConstructor.__init__(self)
+        BaseResolver.__init__(self)
+
+
+PROJECT = {
+    "id": SimpleIdExpr,
+    "owner": SimpleOptStrExpr,
+    "role": SimpleOptStrExpr,
+    "images": None,
+    "volumes": None,
+    "defaults": None,
+}
+
+
+def parse_project_main(ctor: BaseConstructor, node: yaml.MappingNode) -> ast.Project:
+    ret = parse_dict(
+        ctor,
+        node,
+        PROJECT,
+        ast.Project,
+    )
+    return ret
+
+
+def parse_project_defaults(
+    ctor: FlowLoader, node: yaml.MappingNode
+) -> ast.BatchFlowDefaults:
+    return parse_dict(
+        ctor,
+        node,
+        {
+            "tags": ExprOrSeq(StrExpr, type2str),
+            "env": ExprOrMapping(StrExpr, type2str),
+            "volumes": ExprOrSeq(OptStrExpr, type2str),
+            "workdir": OptRemotePathExpr,
+            "life_span": OptTimeDeltaExpr,
+            "preset": OptStrExpr,
+            "schedule_timeout": OptTimeDeltaExpr,
+            "fail_fast": OptBoolExpr,
+            "max_parallel": OptIntExpr,
+            "cache": None,
+        },
+        ast.BatchFlowDefaults,
+    )
+
+
+ProjectLoader.add_path_resolver("project:main", [])  # type: ignore
+ProjectLoader.add_constructor("project:main", parse_project_main)  # type: ignore
+
+ProjectLoader.add_path_resolver(  # type: ignore
+    "project:cache", [(dict, "defaults"), (dict, "cache")]
+)
+ProjectLoader.add_constructor("project:cache", parse_cache)  # type: ignore
+
+ProjectLoader.add_path_resolver(  # type: ignore
+    "project:defaults", [(dict, "defaults")]
+)
+ProjectLoader.add_constructor(  # type: ignore
+    "project:defaults", parse_project_defaults
+)
+
+ProjectLoader.add_path_resolver("project:images", [(dict, "images")])  # type: ignore
+ProjectLoader.add_constructor("project:images", parse_images)  # type: ignore
+
+ProjectLoader.add_path_resolver("project:volumes", [(dict, "volumes")])  # type: ignore
+ProjectLoader.add_constructor("project:volumes", parse_volumes)  # type: ignore
+
+
+def parse_project_stream(stream: TextIO) -> ast.Project:
+    ret: ast.Project
+    loader = ProjectLoader(stream)
+    try:
+        ret = loader.get_single_data()
+        assert isinstance(ret, ast.Project)
+        return ret
+    finally:
+        loader.dispose()  # type: ignore[no-untyped-call]
+
+
+def make_default_project(workspace_stem: str) -> ast.Project:
+    project_id = workspace_stem.replace("-", "_")
+    if not project_id.isidentifier():
+        raise ValueError(
+            f'Workspace directory name "{workspace_stem}" is invalid identifier'
+        )
+    if project_id == project_id.upper():
+        raise ValueError(
+            f'Workspace directory name "{workspace_stem}" is invalid '
+            "identifier, uppercase names are reserved for internal usage"
+        )
+    return ast.Project(
+        _start=Pos(0, 0, LocalPath("<default>")),
+        _end=Pos(0, 0, LocalPath("<default>")),
+        id=SimpleIdExpr(
+            Pos(0, 0, LocalPath("<default>")),
+            Pos(0, 0, LocalPath("<default>")),
+            project_id,
+        ),
+        owner=SimpleOptStrExpr(
+            Pos(0, 0, LocalPath("<default>")),
+            Pos(0, 0, LocalPath("<default>")),
+            None,
+        ),
+        role=SimpleOptStrExpr(
+            Pos(0, 0, LocalPath("<default>")),
+            Pos(0, 0, LocalPath("<default>")),
+            None,
+        ),
+        defaults=None,
+        images=None,
+        volumes=None,
+    )
