@@ -122,6 +122,17 @@ def test_parse_minimal(assets: pathlib.Path) -> None:
         defaults=ast.BatchFlowDefaults(
             _start=Pos(22, 2, config_file),
             _end=Pos(34, 0, config_file),
+            _specified_fields={
+                "env",
+                "fail_fast",
+                "volumes",
+                "life_span",
+                "schedule_timeout",
+                "max_parallel",
+                "preset",
+                "workdir",
+                "tags",
+            },
             tags=SequenceItemsExpr(
                 [
                     StrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), "tag-a"),
@@ -163,10 +174,30 @@ def test_parse_minimal(assets: pathlib.Path) -> None:
             max_parallel=OptIntExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), 10),
             cache=None,
         ),
+        mixins=None,
         tasks=[
             ast.Task(
                 _start=Pos(35, 4, config_file),
                 _end=Pos(57, 0, config_file),
+                _specified_fields={
+                    "life_span",
+                    "http_auth",
+                    "entrypoint",
+                    "title",
+                    "cmd",
+                    "schedule_timeout",
+                    "workdir",
+                    "env",
+                    "tags",
+                    "name",
+                    "preset",
+                    "image",
+                    "http_port",
+                    "pass_config",
+                    "id",
+                    "volumes",
+                },
+                inherits=None,
                 id=OptIdExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), "test_a"),
                 title=OptStrExpr(
                     Pos(0, 0, config_file), Pos(0, 0, config_file), "Batch title"
@@ -175,7 +206,7 @@ def test_parse_minimal(assets: pathlib.Path) -> None:
                 name=OptStrExpr(
                     Pos(0, 0, config_file), Pos(0, 0, config_file), "job-name"
                 ),
-                image=StrExpr(
+                image=OptStrExpr(
                     Pos(0, 0, config_file),
                     Pos(0, 0, config_file),
                     "${{ images.image_a.ref }}",
@@ -288,15 +319,20 @@ def test_parse_seq(assets: pathlib.Path) -> None:
         images=None,
         volumes=None,
         defaults=None,
+        mixins=None,
         tasks=[
             ast.Task(
                 _start=Pos(2, 4, config_file),
                 _end=Pos(6, 2, config_file),
+                _specified_fields={"preset", "cmd", "image"},
+                inherits=None,
                 id=OptIdExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
                 title=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
                 needs=None,
                 name=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
-                image=StrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), "ubuntu"),
+                image=OptStrExpr(
+                    Pos(0, 0, config_file), Pos(0, 0, config_file), "ubuntu"
+                ),
                 preset=OptStrExpr(
                     Pos(0, 0, config_file), Pos(0, 0, config_file), "cpu-micro"
                 ),
@@ -338,11 +374,15 @@ def test_parse_seq(assets: pathlib.Path) -> None:
             ast.Task(
                 _start=Pos(6, 4, config_file),
                 _end=Pos(9, 0, config_file),
+                _specified_fields={"preset", "cmd", "image"},
+                inherits=None,
                 id=OptIdExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
                 title=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
                 needs=None,
                 name=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
-                image=StrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), "ubuntu"),
+                image=OptStrExpr(
+                    Pos(0, 0, config_file), Pos(0, 0, config_file), "ubuntu"
+                ),
                 preset=OptStrExpr(
                     Pos(0, 0, config_file), Pos(0, 0, config_file), "cpu-micro"
                 ),
@@ -410,15 +450,20 @@ def test_parse_needs(assets: pathlib.Path) -> None:
         images=None,
         volumes=None,
         defaults=None,
+        mixins=None,
         tasks=[
             ast.Task(
                 _start=Pos(2, 4, config_file),
                 _end=Pos(7, 2, config_file),
+                _specified_fields={"cmd", "image", "id", "preset"},
+                inherits=None,
                 id=OptIdExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), "task_a"),
                 title=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
                 needs=None,
                 name=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
-                image=StrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), "ubuntu"),
+                image=OptStrExpr(
+                    Pos(0, 0, config_file), Pos(0, 0, config_file), "ubuntu"
+                ),
                 preset=OptStrExpr(
                     Pos(0, 0, config_file), Pos(0, 0, config_file), "cpu-micro"
                 ),
@@ -460,6 +505,8 @@ def test_parse_needs(assets: pathlib.Path) -> None:
             ast.Task(
                 _start=Pos(7, 4, config_file),
                 _end=Pos(11, 0, config_file),
+                _specified_fields={"needs", "image", "cmd", "preset"},
+                inherits=None,
                 id=OptIdExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
                 title=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
                 needs={
@@ -468,7 +515,9 @@ def test_parse_needs(assets: pathlib.Path) -> None:
                     ): NeedsLevel.COMPLETED
                 },
                 name=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
-                image=StrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), "ubuntu"),
+                image=OptStrExpr(
+                    Pos(0, 0, config_file), Pos(0, 0, config_file), "ubuntu"
+                ),
                 preset=OptStrExpr(
                     Pos(0, 0, config_file), Pos(0, 0, config_file), "cpu-micro"
                 ),
@@ -536,15 +585,20 @@ def test_parse_needs_dict(assets: pathlib.Path) -> None:
         images=None,
         volumes=None,
         defaults=None,
+        mixins=None,
         tasks=[
             ast.Task(
                 _start=Pos(2, 4, config_file),
                 _end=Pos(7, 2, config_file),
+                _specified_fields={"preset", "image", "cmd", "id"},
+                inherits=None,
                 id=OptIdExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), "task_a"),
                 title=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
                 needs=None,
                 name=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
-                image=StrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), "ubuntu"),
+                image=OptStrExpr(
+                    Pos(0, 0, config_file), Pos(0, 0, config_file), "ubuntu"
+                ),
                 preset=OptStrExpr(
                     Pos(0, 0, config_file), Pos(0, 0, config_file), "cpu-micro"
                 ),
@@ -586,6 +640,8 @@ def test_parse_needs_dict(assets: pathlib.Path) -> None:
             ast.Task(
                 _start=Pos(7, 4, config_file),
                 _end=Pos(12, 0, config_file),
+                _specified_fields={"preset", "image", "cmd", "needs"},
+                inherits=None,
                 id=OptIdExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
                 title=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
                 needs={
@@ -596,7 +652,9 @@ def test_parse_needs_dict(assets: pathlib.Path) -> None:
                     ): NeedsLevel.RUNNING
                 },
                 name=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
-                image=StrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), "ubuntu"),
+                image=OptStrExpr(
+                    Pos(0, 0, config_file), Pos(0, 0, config_file), "ubuntu"
+                ),
                 preset=OptStrExpr(
                     Pos(0, 0, config_file), Pos(0, 0, config_file), "cpu-micro"
                 ),
@@ -664,13 +722,18 @@ def test_parse_matrix(assets: pathlib.Path) -> None:
         images=None,
         volumes=None,
         defaults=None,
+        mixins=None,
         tasks=[
             ast.Task(
                 _start=Pos(2, 4, config_file),
                 _end=Pos(14, 0, config_file),
+                _specified_fields={"strategy", "image", "cmd"},
+                inherits=None,
                 title=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
                 name=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
-                image=StrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), "ubuntu"),
+                image=OptStrExpr(
+                    Pos(0, 0, config_file), Pos(0, 0, config_file), "ubuntu"
+                ),
                 preset=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
                 schedule_timeout=OptTimeDeltaExpr(
                     Pos(0, 0, config_file), Pos(0, 0, config_file), None
@@ -791,6 +854,7 @@ def test_parse_matrix_with_strategy(assets: pathlib.Path) -> None:
         defaults=ast.BatchFlowDefaults(
             Pos(2, 2, config_file),
             Pos(7, 0, config_file),
+            _specified_fields={"fail_fast", "cache", "max_parallel"},
             tags=None,
             env=None,
             volumes=None,
@@ -815,13 +879,18 @@ def test_parse_matrix_with_strategy(assets: pathlib.Path) -> None:
                 ),
             ),
         ),
+        mixins=None,
         tasks=[
             ast.Task(
                 _start=Pos(8, 4, config_file),
                 _end=Pos(25, 2, config_file),
+                _specified_fields={"image", "strategy", "cmd", "cache"},
+                inherits=None,
                 title=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
                 name=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
-                image=StrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), "ubuntu"),
+                image=OptStrExpr(
+                    Pos(0, 0, config_file), Pos(0, 0, config_file), "ubuntu"
+                ),
                 preset=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
                 schedule_timeout=OptTimeDeltaExpr(
                     Pos(0, 0, config_file), Pos(0, 0, config_file), None
@@ -921,6 +990,8 @@ def test_parse_matrix_with_strategy(assets: pathlib.Path) -> None:
             ast.Task(
                 Pos(25, 4, config_file),
                 Pos(28, 0, config_file),
+                _specified_fields={"id", "image", "cmd"},
+                inherits=None,
                 id=OptIdExpr(
                     Pos(25, 8, config_file), Pos(25, 14, config_file), "simple"
                 ),
@@ -932,7 +1003,7 @@ def test_parse_matrix_with_strategy(assets: pathlib.Path) -> None:
                 cache=None,
                 title=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
                 name=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
-                image=StrExpr(
+                image=OptStrExpr(
                     Pos(26, 11, config_file), Pos(26, 17, config_file), "ubuntu"
                 ),
                 preset=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
@@ -1036,6 +1107,7 @@ def test_parse_args(assets: pathlib.Path) -> None:
         defaults=ast.BatchFlowDefaults(
             _start=Pos(7, 2, config_file),
             _end=Pos(10, 0, config_file),
+            _specified_fields={"tags"},
             tags=SequenceItemsExpr(
                 [
                     StrExpr(
@@ -1068,6 +1140,7 @@ def test_parse_args(assets: pathlib.Path) -> None:
             ),
             cache=None,
         ),
+        mixins=None,
         tasks=[
             ast.Task(
                 _start=Pos(
@@ -1080,9 +1153,13 @@ def test_parse_args(assets: pathlib.Path) -> None:
                     0,
                     config_file,
                 ),
+                _specified_fields={"image", "cmd"},
+                inherits=None,
                 title=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
                 name=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
-                image=StrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), "ubuntu"),
+                image=OptStrExpr(
+                    Pos(0, 0, config_file), Pos(0, 0, config_file), "ubuntu"
+                ),
                 preset=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
                 schedule_timeout=OptTimeDeltaExpr(
                     Pos(0, 0, config_file), Pos(0, 0, config_file), None
@@ -1150,15 +1227,20 @@ def test_parse_enable(assets: pathlib.Path) -> None:
         images=None,
         volumes=None,
         defaults=None,
+        mixins=None,
         tasks=[
             ast.Task(
                 _start=Pos(2, 4, config_file),
                 _end=Pos(6, 2, config_file),
+                _specified_fields={"cmd", "id", "preset", "image"},
+                inherits=None,
                 id=OptIdExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), "task_a"),
                 title=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
                 needs=None,
                 name=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
-                image=StrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), "ubuntu"),
+                image=OptStrExpr(
+                    Pos(0, 0, config_file), Pos(0, 0, config_file), "ubuntu"
+                ),
                 preset=OptStrExpr(
                     Pos(0, 0, config_file), Pos(0, 0, config_file), "cpu-micro"
                 ),
@@ -1200,6 +1282,8 @@ def test_parse_enable(assets: pathlib.Path) -> None:
             ast.Task(
                 _start=Pos(6, 4, config_file),
                 _end=Pos(11, 0, config_file),
+                _specified_fields={"enable", "image", "needs", "cmd", "preset"},
+                inherits=None,
                 id=OptIdExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
                 title=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
                 needs={
@@ -1208,7 +1292,9 @@ def test_parse_enable(assets: pathlib.Path) -> None:
                     ): NeedsLevel.COMPLETED
                 },
                 name=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
-                image=StrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), "ubuntu"),
+                image=OptStrExpr(
+                    Pos(0, 0, config_file), Pos(0, 0, config_file), "ubuntu"
+                ),
                 preset=OptStrExpr(
                     Pos(0, 0, config_file), Pos(0, 0, config_file), "cpu-micro"
                 ),
@@ -1251,6 +1337,183 @@ def test_parse_enable(assets: pathlib.Path) -> None:
     )
 
 
+def test_parse_mixin(assets: pathlib.Path) -> None:
+    workspace = assets
+    config_file = workspace / "batch-mixin.yml"
+    flow = parse_batch(workspace, config_file)
+    assert flow == ast.BatchFlow(
+        Pos(0, 0, config_file),
+        Pos(11, 0, config_file),
+        id=SimpleOptIdExpr(
+            Pos(0, 0, config_file),
+            Pos(0, 0, config_file),
+            None,
+        ),
+        kind=ast.FlowKind.BATCH,
+        title=SimpleOptStrExpr(
+            Pos(0, 0, config_file),
+            Pos(0, 0, config_file),
+            None,
+        ),
+        life_span=OptTimeDeltaExpr(
+            Pos(0, 0, config_file), Pos(0, 0, config_file), None
+        ),
+        params=None,
+        images=None,
+        volumes=None,
+        defaults=None,
+        mixins={
+            "basic": ast.TaskMixin(
+                Pos(3, 4, config_file),
+                Pos(5, 0, config_file),
+                _specified_fields={"image", "preset"},
+                inherits=None,
+                name=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
+                image=OptStrExpr(
+                    Pos(0, 0, config_file),
+                    Pos(0, 0, config_file),
+                    "ubuntu",
+                ),
+                preset=OptStrExpr(
+                    Pos(0, 0, config_file), Pos(0, 0, config_file), "cpu-micro"
+                ),
+                schedule_timeout=OptTimeDeltaExpr(
+                    Pos(0, 0, config_file), Pos(0, 0, config_file), None
+                ),
+                entrypoint=OptStrExpr(
+                    Pos(0, 0, config_file), Pos(0, 0, config_file), None
+                ),
+                cmd=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
+                workdir=OptRemotePathExpr(
+                    Pos(0, 0, config_file), Pos(0, 0, config_file), None
+                ),
+                env=None,
+                volumes=None,
+                tags=None,
+                life_span=OptTimeDeltaExpr(
+                    Pos(0, 0, config_file), Pos(0, 0, config_file), None
+                ),
+                title=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
+                http_port=OptIntExpr(
+                    Pos(0, 0, config_file), Pos(0, 0, config_file), None
+                ),
+                http_auth=OptBoolExpr(
+                    Pos(0, 0, config_file), Pos(0, 0, config_file), None
+                ),
+                pass_config=OptBoolExpr(
+                    Pos(0, 0, config_file), Pos(0, 0, config_file), None
+                ),
+                needs=None,
+                strategy=None,
+                cache=None,
+                enable=EnableExpr(
+                    Pos(0, 0, config_file),
+                    Pos(0, 0, config_file),
+                    "${{ success() }}",
+                ),
+            ),
+        },
+        tasks=[
+            ast.Task(
+                _start=Pos(6, 4, config_file),
+                _end=Pos(9, 2, config_file),
+                _specified_fields={"inherits", "cmd"},
+                inherits=[
+                    StrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), "basic")
+                ],
+                id=OptIdExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
+                title=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
+                needs=None,
+                name=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
+                image=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
+                preset=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
+                schedule_timeout=OptTimeDeltaExpr(
+                    Pos(0, 0, config_file), Pos(0, 0, config_file), None
+                ),
+                entrypoint=OptStrExpr(
+                    Pos(0, 0, config_file), Pos(0, 0, config_file), None
+                ),
+                cmd=OptBashExpr(
+                    Pos(0, 0, config_file), Pos(0, 0, config_file), "echo abc"
+                ),
+                workdir=OptRemotePathExpr(
+                    Pos(0, 0, config_file), Pos(0, 0, config_file), None
+                ),
+                env=None,
+                volumes=None,
+                tags=None,
+                life_span=OptTimeDeltaExpr(
+                    Pos(0, 0, config_file), Pos(0, 0, config_file), None
+                ),
+                http_port=OptIntExpr(
+                    Pos(0, 0, config_file), Pos(0, 0, config_file), None
+                ),
+                http_auth=OptBoolExpr(
+                    Pos(0, 0, config_file), Pos(0, 0, config_file), None
+                ),
+                pass_config=OptBoolExpr(
+                    Pos(0, 0, config_file), Pos(0, 0, config_file), None
+                ),
+                strategy=None,
+                cache=None,
+                enable=EnableExpr(
+                    Pos(0, 0, config_file),
+                    Pos(0, 0, config_file),
+                    "${{ success() }}",
+                ),
+            ),
+            ast.Task(
+                _start=Pos(9, 4, config_file),
+                _end=Pos(11, 0, config_file),
+                _specified_fields={"inherits", "cmd"},
+                inherits=[
+                    StrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), "basic")
+                ],
+                id=OptIdExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
+                title=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
+                needs=None,
+                name=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
+                image=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
+                preset=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
+                schedule_timeout=OptTimeDeltaExpr(
+                    Pos(0, 0, config_file), Pos(0, 0, config_file), None
+                ),
+                entrypoint=OptStrExpr(
+                    Pos(0, 0, config_file), Pos(0, 0, config_file), None
+                ),
+                cmd=OptBashExpr(
+                    Pos(0, 0, config_file), Pos(0, 0, config_file), "echo def"
+                ),
+                workdir=OptRemotePathExpr(
+                    Pos(0, 0, config_file), Pos(0, 0, config_file), None
+                ),
+                env=None,
+                volumes=None,
+                tags=None,
+                life_span=OptTimeDeltaExpr(
+                    Pos(0, 0, config_file), Pos(0, 0, config_file), None
+                ),
+                http_port=OptIntExpr(
+                    Pos(0, 0, config_file), Pos(0, 0, config_file), None
+                ),
+                http_auth=OptBoolExpr(
+                    Pos(0, 0, config_file), Pos(0, 0, config_file), None
+                ),
+                pass_config=OptBoolExpr(
+                    Pos(0, 0, config_file), Pos(0, 0, config_file), None
+                ),
+                strategy=None,
+                cache=None,
+                enable=EnableExpr(
+                    Pos(0, 0, config_file),
+                    Pos(0, 0, config_file),
+                    "${{ success() }}",
+                ),
+            ),
+        ],
+    )
+
+
 def test_batch_job_extra_attrs(assets: pathlib.Path) -> None:
     workspace = assets
     config_file = workspace / "batch-task-extra-attrs.yml"
